@@ -6,6 +6,7 @@ from .. import api
 
 import speakeasy.winenv.defs.nt.ddk as ddk
 import speakeasy.winenv.defs.nt.ntoskrnl as ntos
+import speakeasy.windows.common as winemu
 
 
 class Ntdll(api.ApiHandler):
@@ -90,7 +91,7 @@ class Ntdll(api.ApiHandler):
         hmod = 0
 
         req_lib = self.read_unicode_string(Name)
-        lib = self.normalize_dll_name(req_lib)
+        lib = winemu.normalize_dll_name(req_lib)
 
         hmod = emu.load_library(lib)
 
@@ -212,3 +213,26 @@ class Ntdll(api.ApiHandler):
         rv = Ptr - 1
 
         return rv
+
+    @apihook('NtWaitForSingleObject', argc=3)
+    def NtWaitForSingleObject(self, emu, argv, ctx={}):
+        '''
+        NTSYSAPI
+        NTSTATUS
+        NtWaitForSingleObject(
+            HANDLE         Handle,
+            BOOLEAN        Alertable,
+            PLARGE_INTEGER Timeout
+        );
+        '''
+        hHandle, alertable, timeout = argv
+
+        # Other documented return status are:
+        #      STATUS_TIMEOUT = 0x00000102
+        #      STATUS_ACCESS_DENIED = 0xC0000022
+        #      STATUS_ALERTED = 0x00000101
+        #      STATUS_INVALID_HANDLE = 0xC0000008
+        #      STATUS_USER_APC = 0x000000C0
+        rv = ddk.STATUS_SUCCESS
+
+        return rv
